@@ -36,8 +36,8 @@ class ftpclient:
 	def getLocal_dir(self):
 		return self.local_dir
 
-	def getError_No(self):
-		return self.error_no
+	def getReturnMsg(self):
+		return self.return_msg
 		
 	def getError_Obj(self):
 		return self.error_obj
@@ -55,7 +55,7 @@ class ftpclient:
 	def connect(self):
 		try:
 			self.ftpcon = ftplib.FTP()
-			self.ftpcon.connect(self.hostname,self.portno)
+			self.return_msg = self.ftpcon.connect(self.hostname,self.portno)
 			return C_OK
 		except Exception, e:
 			self.error_obj = e
@@ -64,7 +64,7 @@ class ftpclient:
 	#Loing to FTP Server
 	def login(self):
 		try:
-			self.ftpcon.login(self.user,self.password)
+			self.return_msg = self.ftpcon.login(self.user,self.password)
 			return C_OK
 		except Exception, e:
 			self.error_obj = e
@@ -73,21 +73,30 @@ class ftpclient:
 	#Close Connection
 	def close(self):
 		try:
-			self.ftpcon.quit()
+			self.return_msg = self.ftpcon.quit()
 			return C_OK
 		except Exception, e:
 			try:
-				self.ftpcon.close()
+				self.return_msg = self.ftpcon.close()
 				return C_OK
 			except Exception, e:
 				return C_NG
+	
+	#Abort
+	def abort(self):
+		try:
+			self.return_msg = self.ftpcon.abort()
+			return C_OK
+		except Exception, e:
+			self.error_obj = e
+			return C_NG
 
 	#Get remote dir list
 	def getRemoteDirData(self,pattarn=""):
 		try:
 			self.remotedir_obj = []
 			if pattarn == "":
-				send_command = "LIST"
+				send_command = "LIST -L"
 			else:
 				send_command = "LIST " + pattarn
 			self.ftpcon.retrlines(send_command,self.remotedir_obj.append)
@@ -107,9 +116,9 @@ class ftpclient:
 			return C_NG
 
 	#Change remote dir
-	def changeRemoteDir(self,dirname):
+	def changeRemoteDir(self,dirname,dummy=None):
 		try:
-			self.ftpcon.cwd(dirname)
+			self.return_msg = self.ftpcon.cwd(dirname)
 			self.getRemoteDirData()
 			return C_OK
 		except Exception, e:
@@ -119,7 +128,7 @@ class ftpclient:
 	#Get file(BINARY MODE)
 	def getRemoteFile(self,remote_filename,local_filename):
 		try:
-			self.ftpcon.retrbinary("RETR " + remote_filename,open(local_filename,"wb").write)
+			self.return_msg = self.ftpcon.retrbinary("RETR " + remote_filename,open(local_filename,"wb").write)
 			self.getRemoteDirData()
 			return C_OK
 		except Exception, e:
@@ -131,7 +140,7 @@ class ftpclient:
 	def putLocalFile(self,remote_filename,local_filename):
 		try:
 			localfile = open(local_filename,"rb")
-			self.ftpcon.storbinary("STOR " + remote_filename,localfile)
+			self.return_msg = self.ftpcon.storbinary("STOR " + remote_filename,localfile)
 			localfile.close()
 			self.getRemoteDirData()
 			return C_OK
@@ -150,7 +159,7 @@ class ftpclient:
 	#Create dir
 	def createRemote_dir(self,dir_name):
 		try:
-			self.ftpcon.mkd(dir_name)
+			self.return_msg = self.ftpcon.mkd(dir_name)
 			self.getRemoteDirData()
 			return C_OK
 		except Exception, e:
@@ -160,7 +169,7 @@ class ftpclient:
 	#Remove dir
 	def removeRemote_dir(self,dir_name):
 		try:
-			self.ftpcon.rmd(dir_name)
+			self.return_msg = self.ftpcon.rmd(dir_name)
 			self.getRemoteDirData()
 			return C_OK
 		except Exception, e:
@@ -170,7 +179,7 @@ class ftpclient:
 	#Remove file
 	def removeRemoteFile(self,remote_filename):
 		try:
-			self.ftpcon.delete(remote_filename)
+			self.return_msg = self.ftpcon.delete(remote_filename)
 			self.getRemoteDirData()
 			return C_OK
 		except Exception, e:
@@ -181,7 +190,7 @@ class ftpclient:
 	def changeModeRemoteFile(self,remote_filename,file_mode):
 		try:
 			send_command = "SITE CHMOD %s %s" % (file_mode,remote_filename)
-			self.ftpcon.voidcmd(send_command)
+			self.return_msg = self.ftpcon.sendcmd(send_command)
 			self.getRemoteDirData()
 			return C_OK
 		except Exception, e:
@@ -189,7 +198,7 @@ class ftpclient:
 			return C_NG
 
 	#Get Directory
-	def getRemoteDir(self,remote_dir):
+	def getRemoteDir(self,remote_dir,dummy=None):
 		try:
 			res = self.changeRemoteDir(remote_dir)
 			if res == C_NG:
@@ -202,7 +211,7 @@ class ftpclient:
 			if res == C_NG:
 				return C_NG
 			dir_list = []
-			self.ftpcon.retrlines("LIST",dir_list.append)
+			self.return_msg = self.ftpcon.retrlines("LIST",dir_list.append)
 			for line in dir_list:
 				line_ent = []
 				for ent in string.split(line," "):
@@ -210,10 +219,10 @@ class ftpclient:
 						line_ent.append(ent)
 				if line[0] == "d":
 					if line_ent[8] <> ".." and line_ent[8] <> ".":
-						self.getRemoteDir(line_ent[8])
+						self.return_msg = self.getRemoteDir(line_ent[8])
 				elif line[0] == "-":
 					#print self.ftpcon.pwd() + "/" + line_ent[8]
-					self.getRemoteFile(line_ent[8],line_ent[8])
+					self.return_msg = self.getRemoteFile(line_ent[8],line_ent[8])
 					if res == C_NG:
 						return C_NG
 			res = self.changeRemoteDir("..")
@@ -266,19 +275,40 @@ class ftpclient:
 			self.error_obj = e
 			return C_NG
 	
-	#Set pasv mode
-	def setPasvmode(self):
+	#Change local file name
+	def renameLocalFile(self,fromname,toname):
 		try:
-			self.ftpcon.set_pasv(True)
+			os.rename(fromname,toname)
+			return C_OK
+		except Exception, e:
+			self.error_obj = e
+			return C_NG
+			
+	#Delete local file
+	def deleteLocalFile(self,filename):
+		try:
+			os.remove(filename)
 			return C_OK
 		except Exception, e:
 			self.error_obj = e
 			return C_NG
 
+	#Set pasv mode
+	def setPasvmode(self,mode):
+		try:
+			if mode == C_ON:
+				self.return_msg = self.ftpcon.set_pasv(True)
+			else:
+				self.ftpcon.set_pasv(False)
+			return C_OK
+		except Exception, e:
+			self.error_obj = e
+			return C_NG
+			
 	#Get file(ASCII MODE)
 	def getRemoteFileAscii(self,remote_filename,local_filename):
 		try:
-			self.ftpcon.retrlines("RETR " + remote_filename,open(local_filename,"wb").write)
+			self.return_msg = self.ftpcon.retrlines("RETR " + remote_filename,open(local_filename,"wb").write)
 			self.getRemoteDirData()
 			return C_OK
 		except Exception, e:
@@ -290,7 +320,7 @@ class ftpclient:
 	def putLocalFileAscii(self,remote_filename,local_filename):
 		try:
 			localfile = open(local_filename,"rb")
-			self.ftpcon.storlines("STOR " + remote_filename,localfile)
+			self.return_msg = self.ftpcon.storlines("STOR " + remote_filename,localfile)
 			localfile.close()
 			self.getRemoteDirData()
 			return C_OK
@@ -301,7 +331,7 @@ class ftpclient:
 	#Rename file
 	def renameRemoteFile(self,from_filename,to_filename):
 		try:
-			self.ftpcon.rename(from_filename,to_filename)
+			self.return_msg = self.ftpcon.rename(from_filename,to_filename)
 			return C_OK
 		except Exception, e:
 			self.error_obj = e
