@@ -5,6 +5,8 @@ from ftpy_define import *
 from Tkinter import *
 import tkMessageBox
 import tkSimpleDialog
+import threading
+import time
 
 ###############################################
 #	CUI mode class
@@ -277,11 +279,10 @@ class ftpcui:
 			pass
 
 	#Initial
-	def __init__(self,pacv):
+	def __init__(self,pasv):
 		self.ftpclient = ftpclient.ftpclient()
 		self.connectFlag = C_OFF
-		if pacv == C_OK:
-			self.ftpclient.setPasvmode()
+		self.ftpclient.setPasvmode(pasv)
 
 ###############################################
 #	GUI mode class
@@ -302,8 +303,9 @@ class ftpgui(Frame):
 		self.ftpclient = ftpclient.ftpclient()
 		self.refreshLocalList()
 		self.connectFlag = C_OFF
-		if pasv == C_ON:
-			self.ftpclient.setPasvmode()
+		self.ftpclient.setPasvmode(pasv)
+		self.lockFlag = C_OFF
+		self.ftpypath = os.getcwd()
 			
 	#-----------------------------------------#
 	#          Widget                         #
@@ -311,7 +313,7 @@ class ftpgui(Frame):
 	#Print FTPy widget
 	def printGUI(self):
 		#Tk
-		self.master.title("FTPy-0.1.2")
+		self.master.title("FTPy-0.1.3")
 		
 		#Frame 
 		frameTop = Frame(self)
@@ -345,7 +347,8 @@ class ftpgui(Frame):
 		self.makeMenuDisplay(menuTop)
 		self.makeMenuBookmark(menuTop)
 		self.makeMenuHelp(menuTop)
-		self.makeMenuPopup()
+		self.makeMenuPopupLocal()
+		self.makeMenuPopupRemote()
 		
 	#Menu connect
 	def makeMenuConnect(self,menuTop):
@@ -353,6 +356,7 @@ class ftpgui(Frame):
 		menuTop.add_cascade(label="connect",menu=menuConnect)
 		menuConnect.add_command(label="connect",command=self.connect)			
 		menuConnect.add_command(label="disconnect",command=self.disconnect)
+		menuConnect.add_command(label="abort",command=self.abort)
 		menuConnect.add_command(label="quit",command=self.quit)
 
 	#Menu display
@@ -375,47 +379,57 @@ class ftpgui(Frame):
 		menuHelp.add_command(label="version",command=self.showVersion)
 		self.master.config(menu=menuTop)
 	
+	#Popup menu(Local)
+	def makeMenuPopupLocal(self):
+		self.menuPopupLocal = Menu(font="system 10")
+		self.menuPopupLocal.add_command(label="Create directory",command=self.createLocalDirectory)
+		self.menuPopupLocal.add_command(label="Delete File",command=self.deleteLocalFile)
+		self.menuPopupLocal.add_command(label="Rename",command=self.renameLocalFile)
+		
 	#Popup menu(Remote)
-	def makeMenuPopup(self):
+	def makeMenuPopupRemote(self):
 		self.menuPopupRemote = Menu(font="system 10")
 		self.menuPopupRemote.add_command(label="Change mode",command=self.changeMode)
-		self.menuPopupRemote.add_command(label="Create directory",command=self.createDirectory)
+		self.menuPopupRemote.add_command(label="Create directory",command=self.createRemoteDirectory)
 		self.menuPopupRemote.add_command(label="Delete File",command=self.deleteRemoteFile)
-		self.menuPopupRemote.add_command(label="Rename",command=self.renameFile)
+		self.menuPopupRemote.add_command(label="Rename",command=self.renameRemoteFile)
 
 	#Make host connect infomation widget
 	def makeHostInfoWidget(self,frameTop):
 		self.HostVar = StringVar(); self.HostVar.set("")
 		labelHost = Label(frameTop,text="Hostname",font="system 10")
-		entryHost = Entry(frameTop,width="20",bg="white",font="system 10",textvariable=self.HostVar)
+		self.entryHost = Entry(frameTop,width="20",bg="white",font="system 10",textvariable=self.HostVar)
 		self.PortVar = StringVar(); self.PortVar.set("21")
 		labelPort = Label(frameTop,text="Port",font="system 10")
-		entryPort = Entry(frameTop,width="4",bg="white",font="system 10",textvariable=self.PortVar)
+		self.entryPort = Entry(frameTop,width="4",bg="white",font="system 10",textvariable=self.PortVar)
 		self.UserVar = StringVar(); self.UserVar.set("")
 		labelUser = Label(frameTop,text="User",font="system 10")
-		entryUser = Entry(frameTop,width="10",bg="white",font="system 10",textvariable=self.UserVar)
+		self.entryUser = Entry(frameTop,width="10",bg="white",font="system 10",textvariable=self.UserVar)
 		self.PassVar = StringVar(); self.PassVar.set("")
 		labelPass = Label(frameTop,text="Password",font="system 10")
-		entryPass = Entry(frameTop,width="10",bg="white",font="system 10",textvariable=self.PassVar,show="*")
-		entryHost.bind("<Return>",self.pressEnter); entryPort.bind("<Return>",self.pressEnter)
-		entryUser.bind("<Return>",self.pressEnter); entryPass.bind("<Return>",self.pressEnter)
-		labelHost.grid(row=0,column=1); entryHost.grid(row=0,column=2)
-		labelPort.grid(row=0,column=3); entryPort.grid(row=0,column=4)
-		labelUser.grid(row=0,column=5); entryUser.grid(row=0,column=6)
-		labelPass.grid(row=0,column=7); entryPass.grid(row=0,column=8)
+		self.entryPass = Entry(frameTop,width="10",bg="white",font="system 10",textvariable=self.PassVar,show="*")
+		self.PasvVar = IntVar()
+		chbtnPasv = Checkbutton(frameTop,text="PASV",font="system 10",variable=self.PasvVar,onvalue=C_ON,offvalue=C_OFF)
+		self.entryHost.bind("<Return>",self.pressEnter); self.entryPort.bind("<Return>",self.pressEnter)
+		self.entryUser.bind("<Return>",self.pressEnter); self.entryPass.bind("<Return>",self.pressEnter)
+		labelHost.grid(row=0,column=1); self.entryHost.grid(row=0,column=2)
+		labelPort.grid(row=0,column=3); self.entryPort.grid(row=0,column=4)
+		labelUser.grid(row=0,column=5); self.entryUser.grid(row=0,column=6)
+		labelPass.grid(row=0,column=7); self.entryPass.grid(row=0,column=8)
+		chbtnPasv.grid(row=0,column=9)
 	
 	#Make button widget(connect,disconnect)
 	def makeConnectButton(self,frameTop):
 		btnConnect = Button(frameTop,bitmap="@connect.xbm",width="24",height="24",command=self.connect)
 		btnDisconnect = Button(frameTop,bitmap="error",width="24",height="24",command=self.disconnect)
-		btnConnect.grid(row=0,column=9)
-		btnDisconnect.grid(row=0,column=10)
+		btnConnect.grid(row=0,column=10)
+		btnDisconnect.grid(row=0,column=11)
 
 	#Make local infomation widget
 	def makeLocalListbox(self,frameLocal):
 		#Directory path
 		self.LocalPathVar = StringVar(); self.LocalPathVar.set("")
-		entryLocalPath = Entry(frameLocal,width=40,font="system 10",textvariable=self.LocalPathVar,state=DISABLED)
+		entryLocalPath = Entry(frameLocal,bg="#d9d9d9",width=40,font="system 10",textvariable=self.LocalPathVar,state=DISABLED)
 		entryLocalPath.grid(row=0,column=0,columnspan=2,sticky=W)
 		#Directory list
 		self.listLocal = Listbox(frameLocal,selectmode=EXTENDED,width=40,height=10,bg="white",font="system 10")
@@ -423,22 +437,23 @@ class ftpgui(Frame):
 		scrhLocal = Scrollbar(frameLocal,orient=HORIZONTAL,command=self.listLocal.xview)
 		self.listLocal.config(xscrollcommand=scrhLocal.set,yscrollcommand=scrvLocal.set)
 		self.listLocal.bind("<Double-Button-1>",self.doubleClickLocal)
+		self.listLocal.bind("<ButtonRelease-3>",self.button3ClickLocal)
 		self.listLocal.grid(row=1,column=0)
 		scrvLocal.grid(row=1,column=1,sticky=NS)
 		scrhLocal.grid(row=2,column=0,sticky=EW)
 	
 	#Make update & download button widget
 	def makeUpDlButton(self,frameButton):
-		btnUpdate = Button(frameButton,text="->",font="system 10",command=self.update)
-		btnDownload = Button(frameButton,text="<-",font="system 10",command=self.download)
-		btnUpdate.grid(row=0,column=0)
-		btnDownload.grid(row=1,column=0)
+		self.btnUpdate = Button(frameButton,text="->",font="system 10",command=self.update)
+		self.btnDownload = Button(frameButton,text="<-",font="system 10",command=self.download)
+		self.btnUpdate.grid(row=0,column=0)
+		self.btnDownload.grid(row=1,column=0)
 
 	#Make remote infomation widget
 	def makeRemoteListbox(self,frameRemote):
 		#Directory path
 		self.RemotePathVar = StringVar(); self.RemotePathVar.set("")
-		entryRemotePath = Entry(frameRemote,width=40,font="system 10",textvariable=self.RemotePathVar,state=DISABLED)
+		entryRemotePath = Entry(frameRemote,bg="#d9d9d9",width=40,font="system 10",textvariable=self.RemotePathVar,state=DISABLED)
 		entryRemotePath.grid(row=0,column=0,columnspan=2,sticky=W)
 		#Directory list
 		self.listRemote = Listbox(frameRemote,selectmode=EXTENDED,width=40,height=10,bg="white",font="system 10")
@@ -453,7 +468,7 @@ class ftpgui(Frame):
 
 	#Make application message print widget
 	def makeMessageText(self,frameMessage):
-		self.textMessage = Text(frameMessage,width=88,height=6,font="system 10",state=DISABLED)
+		self.textMessage = Text(frameMessage,bg="#d9d9d9",width=90,height=10,font="system 10",state=DISABLED)
 		scrvMessage = Scrollbar(frameMessage,orient=VERTICAL,command=self.textMessage.yview)
 		self.textMessage.config(yscrollcommand=scrvMessage.set)
 		self.textMessage.grid(row=0,column=0)
@@ -463,26 +478,101 @@ class ftpgui(Frame):
 		self.textMessage.tag_config("blue",foreground="blue")
 	
 	#-----------------------------------------#
-	#          Procedure                      #
+	#          Procedure -Widget Control-     #
+	#-----------------------------------------#
+	#Lock HostInfomation wedgets
+	def lockHostInfomation(self,lockswitch):
+		if lockswitch == C_ON:
+			self.entryHost.config(state=DISABLED,bg="#d9d9d9")
+			self.entryPort.config(state=DISABLED,bg="#d9d9d9")
+			self.entryUser.config(state=DISABLED,bg="#d9d9d9")
+			self.entryPass.config(state=DISABLED,bg="#d9d9d9")
+		else:
+			self.entryHost.config(state=NORMAL,bg="white")
+			self.entryPort.config(state=NORMAL,bg="white")
+			self.entryUser.config(state=NORMAL,bg="white")
+			self.entryPass.config(state=NORMAL,bg="white")
+	
+	#Lock RemoteInfomation wedget
+	def lockRemoteInfomation(self,lockswitch):
+		if lockswitch == C_ON:
+			self.listRemote.config(selectmode=BROWSE,bg="#d9d9d9")
+			self.btnUpdate.config(state=DISABLED)
+			self.btnDownload.config(state=DISABLED)
+		else:
+			self.listRemote.config(selectmode=EXTENDED,bg="white")
+			self.btnUpdate.config(state=NORMAL)
+			self.btnDownload.config(state=NORMAL)
+
+	#-----------------------------------------#
+	#          Procedure -Message-            #
 	#-----------------------------------------#
 	#Print message
 	def printMessage(self,color,msg):
-		self.textMessage.config(state=NORMAL)
-		self.textMessage.insert(END,msg,color)
-		self.textMessage.config(state=DISABLED)
-		self.textMessage.yview(MOVETO,1)
+		if msg <> None:
+			self.textMessage.config(state=NORMAL)
+			self.textMessage.insert(END,msg+"\n",color)
+			self.textMessage.config(state=DISABLED)
+			self.textMessage.yview(MOVETO,1)
 	
 	#Print error massege
 	def printError(self,err_obj):
-		for e in err_obj:
-			self.printMessage(C_RED,e)
-		self.printMessage(C_BLACK,"\n")
+		string_tmp = "abc"
+		if type(string_tmp) == type(err_obj):
+			self.printMessage(C_RED,err_obj)
+		else:
+			for e in err_obj:
+				self.printMessage(C_RED,e)
+			self.printMessage(C_BLACK,"\n")
+			
 	
+	#-----------------------------------------#
+	#          Procedure -Thread-             #
+	#-----------------------------------------#
+	def threading(self,method_name=None,method_args=(),obj_size=0):
+		while self.lockFlag == C_ON:
+			time.sleep(0.3)
+		self.lockFlag = C_ON
+		thread_connect = threading.Thread(target=method_name,args=method_args)
+		thread_connect.start()
+		thread_wait = threading.Thread(target=self.waitThread,args=(thread_connect,obj_size))
+		thread_wait.start()
+		return C_OK
+		
+	def waitThread(self,thread_obj,obj_size):
+		self.lockRemoteInfomation(C_ON)
+		stime = ftime = time.time()
+		while thread_obj.isAlive():
+			ntime = time.time()
+			if ntime - ftime > 1:
+				self.textMessage.config(state=NORMAL)
+				self.textMessage.insert(END,".")
+				self.textMessage.config(state=DISABLED)
+				ftime = ntime
+				time.sleep(1)
+		etime = time.time() - stime
+		if obj_size <> 0:
+			if obj_size / etime > 1000:
+				bps = str(int((obj_size / etime) / 1000))+"Kbps"
+			else:
+				bps = str(int(obj_size / etime))+"bps"
+			self.printMessage(C_BLACK," "+str(int(etime))+"sec("+bps+")")
+		else:
+			self.printMessage(C_BLACK," "+str(int(etime))+"sec")
+		self.textMessage.config(state=NORMAL)
+		self.printMessage(C_BLUE,self.ftpclient.getReturnMsg())
+		self.textMessage.config(state=DISABLED)
+		self.lockRemoteInfomation(C_OFF)
+		self.lockFlag = C_OFF
+		
+	#-----------------------------------------#
+	#          Procedure -Connection-         #
+	#-----------------------------------------#
 	#Connecting to FTP server
 	def connect(self):
 		#Check connection
 		if self.connectFlag == C_ON:
-			self.printMessage(C_RED,"Already connected , use close first\n")
+			self.printError("Already connected , use close first")
 			return C_NG
 		#Get connecting parameters
 		hostname	= self.HostVar.get()
@@ -493,10 +583,10 @@ class ftpgui(Frame):
 			user = "anonymous"
 		#Check connection parameters
 		if hostname == "":
-			self.printMessage(C_RED,"Please input hostname\n")
+			self.printError("Please input hostname")
 			return C_NG
 		if portno == "":
-			self.printMessage(C_RED,"Please input portno\n")
+			self.printError("Please input portno")
 			return C_NG
 		#Set connection parameters
 		self.ftpclient.setHostname(hostname)
@@ -504,29 +594,37 @@ class ftpgui(Frame):
 		self.ftpclient.setUser(user)
 		self.ftpclient.setPassword(password)
 		#Check hostname
-		self.printMessage(C_BLACK,"Check Hostname(" + hostname + ")..")
+		self.printMessage(C_BLACK,"Check Hostname(" + hostname + ")")
 		res = self.checkHostname(hostname)
 		if res <> C_OK:
-			self.printMessage(C_RED,hostname + " is not found\n")
+			self.printError(hostname + " is not found")
 			return C_NG
 		else:
-			self.printMessage(C_BLUE,"OK\n")
+			self.printMessage(C_BLUE,"OK")
+		#Check Pasv mode
+		self.ftpclient.setPasvmode(self.PasvVar.get())
+		self.printMessage(C_BLACK,"Pasv mode")
+		if self.PasvVar.get() == C_ON:
+			self.printMessage(C_BLUE,"ON")
+		else:
+			self.printMessage(C_BLUE,"OFF")
 		#Connecting FTP server start
-		self.printMessage(C_BLACK,"Connect(" + hostname + ":" + portno + ")..")
+		self.printMessage(C_BLACK,"Connect(" + hostname + ":" + portno + ")")
 		res = self.ftpclient.connect()
 		if res == C_OK:
-			self.printMessage(C_BLUE,"OK\n")
+			self.printMessage(C_BLUE,self.ftpclient.getReturnMsg())
 			self.connectFlag = C_ON
 			#Login to FTP server
-			self.printMessage(C_BLACK,"Login(" + user + ")..")
+			self.printMessage(C_BLACK,"Login(" + user + ")")
 			res = self.ftpclient.login()
 			if res == C_OK:
+				self.lockHostInfomation(C_ON)
+				self.printMessage(C_BLUE,self.ftpclient.getReturnMsg())
 				self.refreshRemoteList()
-				self.printMessage(C_BLUE,"OK\n")
 			else:
-				self.printMessage(C_RED,"Failure\n")
+				self.printError(self.ftpclient.getError_Obj())
 		else:
-			self.printMessage(C_RED,"Failure\n")
+			self.printError(self.ftpclient.getError_Obj())
 
 	#Check hostname
 	def checkHostname(self,hostname):
@@ -539,28 +637,52 @@ class ftpgui(Frame):
 	#Disconnect to FTP server
 	def disconnect(self):
 		if self.connectFlag == C_ON:
-			self.printMessage(C_BLACK,"Disconnect..")
+			self.printMessage(C_BLACK,"Disconnect")
 			res = self.ftpclient.close()
 			if res <> C_OK:
-				self.printMessage(C_RED,"Failure\n")
+				self.printError(self.ftpclient.getError_Obj())
 				return C_NG
 			#clear to remote widget
+			self.lockHostInfomation(C_OFF)
 			self.listRemote.delete(0,END)
 			self.RemotePathVar.set("")
 			self.connectFlag = C_OFF
-			self.printMessage(C_BLUE,"OK\n")
+			if self.ftpclient.getReturnMsg() == None:
+				self.printError("999 Timeout: closing control connection")
+			else:
+				self.printMessage(C_BLUE,self.ftpclient.getReturnMsg())
 
+	#Abort
+	def abort(self):
+		if self.connectFlag == C_ON:
+			self.printMessage(C_BLACK,"ABORT")
+			res = self.ftpclient.abort()
+			if res <> C_OK:
+				self.printError(self.ftpclient.getError_Obj())
+				return C_NG
+			self.printMessage(C_BLUE,self.ftpclient.getReturnMsg())
+	
 	#FTPy quit
 	def quit(self):
 		self.disconnect()
 		self.master.destroy()
 
+	#-----------------------------------------#
+	#          Procedure -Help-               #
+	#-----------------------------------------#
 	#Output version infomation
 	def showVersion(self):
-		tkMessageBox.showinfo(title="Version",message="FTPy\nVersion 0.1.2\nCreater:shinkichi")
+		tkMessageBox.showinfo(title="Version",message="FTPy\nVersion 0.1.3\nCreater:shinkichi")
 	
+	#-----------------------------------------#
+	#          Procedure -File Transfe-       #
+	#-----------------------------------------#
 	#File update(Local -> Remote)
 	def update(self):
+		work_thread = threading.Thread(target=self.threadUpdate)
+		work_thread.start()
+	
+	def threadUpdate(self):
 		#Get select file(s)
 		select_indexes = self.listLocal.curselection()
 		if len(select_indexes) == 0:
@@ -569,16 +691,27 @@ class ftpgui(Frame):
 			item = self.listLocal.get(index)
 			if item[0] == "-":	#File
 				filename = item[string.rfind(item," ")+1:]
-				self.printMessage(C_BLACK,"Update(" + filename + ")..")
-				res = self.ftpclient.putLocalFile(filename,filename)
+				filesize = long(item[1:13])
+				res = self.threading(method_name=self.ftpclient.putLocalFile,method_args=(filename,filename),obj_size=filesize)
 				if res == C_OK:
-					self.printMessage(C_BLUE,"OK\n")
-					self.refreshRemoteList()
+					if self.LocalPathVar.get() == "/":
+						tmpPath = self.LocalPathVar.get()+filename
+					else:
+						tmpPath = self.LocalPathVar.get()+"/"+filename
+					self.printMessage(C_BLACK,"STOR "+tmpPath)
 				else:
-					self.printMessage(C_RED,"Failure\n")
-	
+					self.printError(self.ftpclient.getError_Obj())
+			while self.lockFlag == C_ON:
+				time.sleep(0.5)
+		time.sleep(0.5)
+		self.refresh()
+
 	#File download(Remote -> Local)
 	def download(self):
+		work_thread = threading.Thread(target=self.threadDownload)
+		work_thread.start()
+	
+	def threadDownload(self):
 		#Get select file(s)
 		select_indexes = self.listRemote.curselection()
 		if len(select_indexes) == 0:
@@ -586,28 +719,38 @@ class ftpgui(Frame):
 		for index in select_indexes:
 			item = self.listRemote.get(index)
 			objname = item[string.rfind(item," ")+1:]
-			self.printMessage(C_BLACK,"Download(" + objname + ")..")
 			if item[0] == "-":	#File
-				res = self.ftpclient.getRemoteFile(objname,objname)
+				filesize = long(item[10:22])
+				res = self.threading(method_name=self.ftpclient.getRemoteFile,method_args=(objname,objname),obj_size=filesize)
 			elif item[0] == "d":	#Directory
-				res = self.ftpclient.getRemoteDir(objname)
+				res = self.threading(method_name=self.ftpclient.getRemoteDir,method_args=(objname,"dummy"))
 			if res == C_OK:
-				self.printMessage(C_BLUE,"OK\n")
-				self.refreshLocalList()
+				if self.RemotePathVar.get() == "/":
+					tmpPath = self.RemotePathVar.get()+objname
+				else:
+					tmpPath = self.RemotePathVar.get()+"/"+objname
+				self.printMessage(C_BLACK,"RETR "+tmpPath)
 			else:
-				self.printMessage(C_RED,"Failure\n")
-
+				self.printError(self.ftpclient.getError_Obj())
+		while self.lockFlag == C_ON:
+			time.sleep(0.5)
+		self.refresh()
+			
+	#-----------------------------------------#
+	#          Procedure -Directory-          #
+	#-----------------------------------------#
 	#Refresh dirctory list(Local,Remote)
 	def refresh(self):
 		self.refreshLocalList()
-		self.refreshRemoteList()
+		if self.lockFlag == C_OFF:
+			self.refreshRemoteList()
 
 	#Refresh local directory list
 	def refreshLocalList(self):
 		#Call getCurrentLocalDir method
 		res = self.ftpclient.getCurrentLocalDir()
 		if res <> C_OK:
-			self.printMessage(C_BLACK,"Connection..Failure\n")
+			self.printError(self.ftpclient.getError_Obj())
 			return C_NG
 		#Get local current directory
 		current_dir = self.ftpclient.getMsg_Obj()
@@ -616,7 +759,7 @@ class ftpgui(Frame):
 		self.listLocal.delete(0,END)
 		res = self.ftpclient.getLocalDirData(current_dir[0])
 		if res <> C_OK:
-			self.printMessage(C_RED,"Connection..Failure\n")
+			self.printError(self.ftpclient.getError_Obj())
 			return C_NG
 		#Get local directory data
 		dir_obj = []
@@ -643,7 +786,7 @@ class ftpgui(Frame):
 		#Get remote current directory
 		res = self.ftpclient.getCurrentRemoteDir()
 		if res <> C_OK:
-			self.printMessage(C_RED,"Connection..Failure\n")
+			self.printError(self.ftpclient.getError_Obj())
 			return C_NG
 		current_dir = self.ftpclient.getMsg_Obj()
 		self.RemotePathVar.set(current_dir[0])
@@ -652,7 +795,7 @@ class ftpgui(Frame):
 		#Get remote directory data 
 		res = self.ftpclient.getRemoteDirData()
 		if res <> C_OK:
-			self.printMessage(C_RED,"Connection..Failure\n")
+			self.printError(self.ftpclient.getError_Obj())
 			return C_NG
 		dir_obj = []
 		dir_obj = self.ftpclient.getRemoteDir_Obj()
@@ -665,10 +808,7 @@ class ftpgui(Frame):
 				line_ent = self.splitDirEntry(e)
 				permission = line_ent[0] 
 				filename = line_ent[len(line_ent)-1]
-				if e[0] == "-":
-					filesize = str(self.ftpclient.getRemoteFileSize(filename))
-				else:
-					filesize = "4096"
+				filesize = line_ent[4]
 				if len(filesize) < 10:
 					add_space = " " * (11 - len(filesize))
 					filesize = add_space + filesize
@@ -683,139 +823,224 @@ class ftpgui(Frame):
 				line_ent.append(e)
 		return line_ent
 
+	#-----------------------------------------#
+	#          Procedure -Remote Control-     #
+	#-----------------------------------------#
 	#Change mode(Remote)
 	def changeMode(self):
-		indexed = self.listRemote.curselection()
-		if len(indexed) == 0:
+		select_indexes = self.listRemote.curselection()
+		if len(select_indexes) == 0:
 			return C_NG
-		item = self.listRemote.get(indexed[0])
-		#Check item
-		if item[0] == "-" or item[0] == "d":	#File,Dirctory
-			objname = item[string.rfind(item," ")+1:]
 		mode = tkSimpleDialog.askstring("Change Mode","Mode\nexampl:744 +x")
-		if mode <> None:
-			self.printMessage(C_BLACK,"chmod(" + mode + ")..")
-			res = self.ftpclient.changeModeRemoteFile(objname,mode)
-			if res == C_OK:
-				self.printMessage(C_BLUE,"OK\n")
-				self.refreshRemoteList()
-			else:
-				self.printError(self.ftpclient.getError_Obj())
-				return C_NG
+		if mode == None:
+			return C_NG
+		for index in select_indexes:
+			item = self.listRemote.get(index)
+			#Check item
+			if item[0] == "-" or item[0] == "d":	#File,Dirctory
+				objname = item[string.rfind(item," ")+1:]
+				self.printMessage(C_BLACK,"SITE CHMOD "+mode+" "+objname)
+				res = self.ftpclient.changeModeRemoteFile(objname,mode)
+				if res == C_OK:
+					self.printMessage(C_BLUE,self.ftpclient.getReturnMsg())
+					self.refreshRemoteList()
+				else:
+					self.printError(self.ftpclient.getError_Obj())
+					return C_NG
 
 	#Create directory(Remote)
-	def createDirectory(self):
+	def createRemoteDirectory(self):
 		dirname = tkSimpleDialog.askstring("Create Directory","Directory Name")
 		if dirname <> None:
-			self.printMessage(C_BLACK,"mkdir(" + dirname + ")..")
+			self.printMessage(C_BLACK,"MKD "+dirname)
 			res = self.ftpclient.createRemote_dir(dirname)
 			if res == C_OK:
-				self.printMessage(C_BLUE,"OK\n")
+				self.printMessage(C_BLUE,self.ftpclient.getReturnMsg())
 				self.refreshRemoteList()
 			else:
 				self.printError(self.ftpclient.getError_Obj())
 				return C_NG
 				
-
-	#Double click event(Remote)
-	def doubleClickRemote(self,event):
-		#Get select item
-		indexed = self.listRemote.curselection()
-		if len(indexed) == 0:
-			return C_NG
-		item = self.listRemote.get(indexed[0])
-		#Check item
-		if item[0] == "d":	#Directory
-			dirname = item[string.rfind(item," ")+1:]
-			self.printMessage(C_BLACK,"cd(" + dirname + ")..")
-			#Change remote directory
-			res = self.ftpclient.changeRemoteDir(dirname)
-			if res <> C_OK:
-				self.printMessage(C_RED,"Connection..Failure\n")
-				return C_NG
-			self.printMessage(C_BLUE,"OK\n")
-			self.refreshRemoteList()
-
-	#Double click event(Local)
-	def doubleClickLocal(self,event):
-		#Get select item
-		indexed = self.listLocal.curselection()
-		if len(indexed) == 0:
-			return C_NG
-		item = self.listLocal.get(indexed[0])
-		#Check item
-		if item[0] == "d":	#Directory
-			dirname = item[string.rfind(item," ")+1:]
-			self.printMessage(C_BLACK,"lcd(" + dirname + ")..")
-			#Change local directory
-			res = self.ftpclient.changeLocalDir(dirname)
-			if res <> C_OK:
-				self.printMessage(C_RED,"Connection..Failure\n")
-				return C_NG
-			self.printMessage(C_BLUE,"OK\n")
-			self.refreshLocalList()
-
 	#Delete remote file
 	def deleteRemoteFile(self):
 		#Get select item
-		indexed = self.listRemote.curselection()
-		if len(indexed) == 0:
+		select_indexes = self.listRemote.curselection()
+		if len(select_indexes) == 0:
 			return C_NG
-		selectlist = list(indexed)
+		selectlist = list(select_indexes)
 		selectlist.reverse()
-		for e in selectlist:
-			item = self.listRemote.get(e)
+		for index in selectlist:
+			item = self.listRemote.get(index)
 			#Check item
 			if item[0] == "-":	#File
 				filename = item[string.rfind(item," ")+1:]
-				self.printMessage(C_BLACK,"rm(" + filename + ")..")
+				self.printMessage(C_BLACK,"RMD "+filename)
 				#Delete file
 				res = self.ftpclient.removeRemoteFile(filename)
 				if res <> C_OK:
-					self.printMessage(C_RED,"Connection..Failure\n")
+					self.printError(self.ftpclient.getError_Obj())
 					return C_NG
-				self.printMessage(C_BLUE,"OK\n")
+				self.printMessage(C_BLUE,self.ftpclient.getReturnMsg())
 				self.refreshRemoteList()
 			
 
 	#Rename remote file,object
-	def renameFile(self):
+	def renameRemoteFile(self):
 		#Get select item
-		indexed = self.listRemote.curselection()
-		if len(indexed) == 0:
+		select_indexes = self.listRemote.curselection()
+		if len(select_indexes) <> 1:
 			return C_NG
-		item = self.listRemote.get(indexed[0])
+		item = self.listRemote.get(select_indexes[0])
 		#Check item
 		if item[0] == "d" or item[0] == "-":	#Directory or File
 			fromname = item[string.rfind(item," ")+1:]
 			toname = tkSimpleDialog.askstring("Rename","To filename")
-			if toname <> "":
-				self.printMessage(C_BLACK,"rename(" + fromname + " -> " + toname + ")..")
+			if toname <> None:
+				self.printMessage(C_BLACK,"RNFR "+fromname)
+				self.printMessage(C_BLACK,"RNTO "+toname)
 				res = self.ftpclient.renameRemoteFile(fromname,toname)
 				if res <> C_OK:
-					self.printMessage(C_RED,"Connection..Failure\n")
+					self.printError(self.ftpclient.getError_Obj())
 					return C_NG
-				self.printMessage(C_BLUE,"OK\n")
+				self.printMessage(C_BLUE,self.ftpclient.getReturnMsg())
 				self.refreshRemoteList()
 
+	#-----------------------------------------#
+	#          Procedure -Local Control-      #
+	#-----------------------------------------#
+	#Create directory(Local)
+	def createLocalDirectory(self):
+		dirname = tkSimpleDialog.askstring("Create Directory","Directory Name")
+		if dirname <> None:
+			self.printMessage(C_BLACK,"mkdir(" + dirname + ")")
+			res = self.ftpclient.createLocalDir(dirname)
+			if res == C_OK:
+				self.printMessage(C_BLUE,"OK")
+				self.refreshLocalList()
+			else:
+				self.printError(self.ftpclient.getError_Obj())
+				return C_NG
+				
+	#Delete file(Local)
+	def deleteLocalFile(self):
+		#Get select item
+		select_indexes = self.listLocal.curselection()
+		if len(select_indexes) == 0:
+			return C_NG
+		selectlist = list(select_indexes)
+		selectlist.reverse()
+		for index in selectlist:
+			item = self.listLocal.get(index)
+			#Check item
+			if item[0] == "-":	#File
+				filename = item[string.rfind(item," ")+1:]
+				self.printMessage(C_BLACK,"rm(" + filename + ")")
+				#Delete file
+				res = self.ftpclient.deleteLocalFile(filename)
+				if res <> C_OK:
+					self.printError(self.ftpclient.getError_Obj())
+					return C_NG
+				self.printMessage(C_BLUE,"OK")
+				self.refreshLocalList()
+			
 
+	#Rename file,object(Local)
+	def renameLocalFile(self):
+		#Get select item
+		select_indexes = self.listLocal.curselection()
+		if len(select_indexes) <> 1:
+			return C_NG
+		item = self.listLocal.get(select_indexes[0])
+		#Check item
+		if item[0] == "d" or item[0] == "-":	#Directory or File
+			fromname = item[string.rfind(item," ")+1:]
+			toname = tkSimpleDialog.askstring("Rename","To filename")
+			if toname <> None:
+				self.printMessage(C_BLACK,"rename(" + fromname + " -> " + toname + ")")
+				res = self.ftpclient.renameLocalFile(fromname,toname)
+				if res <> C_OK:
+					self.printError(self.ftpclient.getError_Obj())
+					return C_NG
+				self.refreshLocalList()
+				self.printMessage(C_BLUE,"OK")
+
+	#-----------------------------------------#
+	#          Procedure -Event Remote-       #
+	#-----------------------------------------#
+	#Double click event(Remote)
+	def doubleClickRemote(self,event):
+		if self.lockFlag == C_ON:
+			return C_NG
+		#Get select item
+		select_indexes = self.listRemote.curselection()
+		if len(select_indexes) <> 1:
+			return C_NG
+		item = self.listRemote.get(select_indexes[0])
+		#Check item
+		if item[0] == "d":	#Directory
+			dirname = item[string.rfind(item," ")+1:]
+			self.printMessage(C_BLACK,"CWD "+dirname)
+			#Change remote directory
+			res = self.ftpclient.changeRemoteDir(dirname)
+			if res == C_OK:
+				self.printMessage(C_BLUE,self.ftpclient.getReturnMsg())
+			else:
+				self.printError(self.ftpclient.getError_Obj())
+				return C_NG
+		self.refreshRemoteList()
+
+	#Button3 Click(Remote)
+	def button3ClickRemote(self,event):
+		if self.lockFlag == C_ON:
+			return C_NG
+		self.menuPopupRemote.tk_popup(event.x_root,event.y_root)
+
+	#-----------------------------------------#
+	#          Procedure -Event Local-        #
+	#-----------------------------------------#
+	#Double click event(Local)
+	def doubleClickLocal(self,event):
+		#Get select item
+		select_indexes = self.listLocal.curselection()
+		if len(select_indexes) == 0:
+			return C_NG
+		item = self.listLocal.get(select_indexes[0])
+		#Check item
+		if item[0] == "d":	#Directory
+			dirname = item[string.rfind(item," ")+1:]
+			self.printMessage(C_BLACK,"lcd(" + dirname + ")")
+			#Change local directory
+			res = self.ftpclient.changeLocalDir(dirname)
+			if res <> C_OK:
+				self.printError(self.ftpclient.getError_Obj())
+				return C_NG
+			self.printMessage(C_BLUE,"OK")
+			self.refreshLocalList()
+
+	#Button3 Click(Local)
+	def button3ClickLocal(self,event):
+		self.menuPopupLocal.tk_popup(event.x_root,event.y_root)
+
+	#-----------------------------------------#
+	#          Procedure -Event Common-       #
+	#-----------------------------------------#
 	#Entry press
 	def pressEnter(self,event):
 		self.connect()
 
-	#Button3 Click(Remote)
-	def button3ClickRemote(self,event):
-		self.menuPopupRemote.tk_popup(event.x_root,event.y_root)
-
+	#-----------------------------------------#
+	#          Procedure -Bookmark Control-   #
+	#-----------------------------------------#
 	#Add Bookmark
 	def addBookmark(self):
 		try:
-			fileBookmark = open("bookmark","a")
+			fileBookmark = open(os.path.join(self.ftpypath,"bookmark"),"a")
 			host = self.HostVar.get()
 			port = self.PortVar.get()
 			user = self.UserVar.get()
 			passwd = self.PassVar.get()
-			item = host + ":" + port + ":" + user + ":" + passwd + "\n"
+			pasv = self.PasvVar.get()
+			item = host + ":" + port + ":" + user + ":" + passwd + ":" + str(pasv) + "\n"
 			fileBookmark.write(item)
 			fileBookmark.close()
 		except Exception, e:
@@ -824,7 +1049,7 @@ class ftpgui(Frame):
 	#Open Bookmark
 	def openBookmark(self):
 		wOpenBookmark = bookmark()
-		wOpenBookmark.init()
+		wOpenBookmark.init(self.ftpypath)
 		self.wait_window(wOpenBookmark)
 		result = wOpenBookmark.selectItem
 		if result <> "":
@@ -832,7 +1057,8 @@ class ftpgui(Frame):
 			self.HostVar.set(item[0])	
 			self.PortVar.set(item[1])	
 			self.UserVar.set(item[2])	
-			self.PassVar.set(item[3])	
+			self.PassVar.set(item[3])
+			self.PasvVar.set(int(item[4]))
 		
 
 	
@@ -842,7 +1068,8 @@ class ftpgui(Frame):
 class bookmark(Toplevel):
 
 	#Class initial
-	def init(self):
+	def init(self,ftpypath):
+		self.ftpypath = ftpypath
 		master=None
 		Toplevel.__init__(self,master)
 		self.printBookmark()
@@ -905,7 +1132,7 @@ class bookmark(Toplevel):
 	def readBookmark(self):
 		try:	
 			self.bookmarks = []
-			fileBookmark = open("bookmark","r")
+			fileBookmark = open(os.path.join(self.ftpypath,"bookmark"),"r")
 			self.bookmarks = fileBookmark.readlines()
 			fileBookmark.close()
 			return self.bookmarks
@@ -919,7 +1146,7 @@ class bookmark(Toplevel):
 	#Write bookmark file
 	def writeBookmark(self):
 		try:
-			fileBookmark = open("bookmark","w")
+			fileBookmark = open(os.path.join(self.ftpypath,"bookmark"),"w")
 			for e in self.bookmarks:
 				fileBookmark.write(e)
 			fileBookmark.close()
@@ -936,6 +1163,10 @@ class bookmark(Toplevel):
 				item = []
 				item = string.split(e,":")
 				item[3] = "******"
+				if int(item[4][0]) == C_ON:
+					item[4] = "PASV"
+				else:
+					item[4] = "NO PASV"
 				self.listBookmark.insert(END,string.join(item,":"))
 	
 	#Popup Menu
